{
   Copyright (C) 2010-2014 MegaBrutal

   This unit is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This unit is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
}


{$MODE DELPHI}
unit DNSMX;

(*

   For reference, here are the C declarations of the used
   functions and structures:

   DNS_STATUS WINAPI DnsQuery(
     __in         PCTSTR lpstrName,
     __in         WORD wType,
     __in         DWORD Options,
     __inout_opt  PVOID pExtra,
     __out_opt    PDNS_RECORD *ppQueryResultsSet,
     __out_opt    PVOID *pReserved
   );

   void WINAPI DnsRecordListFree(
     __inout_opt  PDNS_RECORD pRecordList,
     __in         DNS_FREE_TYPE FreeType
   );

   typedef struct _DnsRecord {
     DNS_RECORD *pNext;
     PWSTR      pName;
     WORD       wType;
     WORD       wDataLength;
     union {
       DWORD            DW;
       DNS_RECORD_FLAGS S;
     } Flags;
     DWORD      dwTtl;
     DWORD      dwReserved;
     union {
       DNS_MX_DATA     MX, Mx, AFSDB, Afsdb, RT, Rt;
     } Data;
   } DNS_RECORD, *PDNS_RECORD;

   typedef struct {
     PWSTR pNameExchange;
     WORD  wPreference;
     WORD  Pad;
   } DNS_MX_DATA, *PDNS_MX_DATA;

*)

interface
uses SysUtils, Classes;

const

   DNS_TYPE_MX = $000F;

type

   pWideChar = pChar;

   DnsMXData = packed record
      pNameExchange: pWideChar;
      wPreference: word;
      Pad: word;
   end;

   ppDnsRecord = ^pDnsRecord;
   pDnsRecord = ^DnsRecord;
   DnsRecord = packed record
      pNext: pDnsRecord;
      pName: pWideChar;
      wType: word;
      wDataLength: word;
      Flags: longword;
      dwTtl: longword;
      dwReserved: longword;
      Data: DnsMXData;
   end;

   { Better to have a Pascal-format DnsMXData: }
   TDNSMXRecord = record
      Preference: word;
      HostName: string;
   end;

   TDNSMXRecordArray = array of TDNSMXRecord;


   function DnsQuery(lsstrName: PChar; wType: word; Options: longword;
      pExtra: pointer; ppQueryResultsSet: ppDnsRecord; pReserved: pointer): longword; stdcall;
      external 'dnsapi.dll' name 'DnsQuery_A';

   procedure DnsRecordListFree(pRecordList: pDnsRecord; FreeType: word); stdcall;
      external 'dnsapi.dll' name 'DnsRecordListFree';


   function GetMXRecordArray(HostName: string; var ResultArray: TDNSMXRecordArray): boolean;
   procedure SortMXRecordArray(var DNSMXRecordArray: TDNSMXRecordArray);
   function MakeMXRecordList(const DNSMXRecordArray: TDNSMXRecordArray): TStrings;
   function GetCorrectMXRecordList(HostName: string): TStrings;



implementation


function GetMXRecordArray(HostName: string; var ResultArray: TDNSMXRecordArray): boolean;
var P, N: pDnsRecord; DNSMXRecord: TDNSMXRecord; ap: integer;
begin
   P:= nil;
   DnsQuery(PChar(HostName), DNS_TYPE_MX, 0, nil, @P, nil);
   N:= P;
   while N <> nil do begin
      if N^.wType = DNS_TYPE_MX then begin
         DNSMXRecord.Preference:= N^.Data.wPreference;
         DNSMXRecord.HostName:= N^.Data.pNameExchange;
         ap:= Length(ResultArray);
         SetLength(ResultArray, ap + 1);
         ResultArray[ap]:= DNSMXRecord;
      end;
      N:= N^.pNext;
   end;
   if P <> nil then DnsRecordListFree(P, 0);
   Result:= Length(ResultArray) <> 0;
end;

procedure SortMXRecordArray(var DNSMXRecordArray: TDNSMXRecordArray);
var i, j: integer; T: TDNSMXRecord;
begin
   for i:= 1 to Length(DNSMXRecordArray) - 1 do begin
      T:= DNSMXRecordArray[i];
      j:= i;
      while (j > 0) and (DNSMXRecordArray[j-1].Preference > T.Preference) do begin
         DNSMXRecordArray[j]:= DNSMXRecordArray[j-1];
         Dec(j);
      end;
      DNSMXRecordArray[j]:= T;
   end;
end;

function MakeMXRecordList(const DNSMXRecordArray: TDNSMXRecordArray): TStrings;
var i: integer;
begin
   Result:= TStringList.Create;
   for i:= 0 to Length(DNSMXRecordArray) - 1 do
      Result.Add(DNSMXRecordArray[i].HostName);
end;

function GetCorrectMXRecordList(HostName: string): TStrings;
var DNSMXRecordArray: TDNSMXRecordArray;
begin
   SetLength(DNSMXRecordArray, 0);
   if GetMXRecordArray(HostName, DNSMXRecordArray) then begin
      SortMXRecordArray(DNSMXRecordArray);
      Result:= MakeMXRecordList(DNSMXRecordArray);
   end
   else begin
      { If the domain has no MX record, the A record shall be used. }
      Result:= TStringList.Create;
      Result.Add(HostName);
   end;
end;


end.
